@@ -2,13 +2,10 @@
 
 from __future__ import annotations
 
-import logging
-
-_LOGGER = logging.getLogger(__name__)
-
 import asyncio
 import base64
 import json
+import logging
 import ssl
 import urllib.parse
 from pathlib import Path
@@ -18,8 +15,10 @@ import aiohttp
 
 from asusrouter import (
     AsusRouter404,
+    AsusRouterAuthorizationError,
     AsusRouterConnectionError,
     AsusRouterConnectionTimeoutError,
+    AsusRouterError,
     AsusRouterLoginBlockError,
     AsusRouterLoginError,
     AsusRouterResponseError,
@@ -41,6 +40,8 @@ from asusrouter.const import (
 )
 from asusrouter.util import converters, parsers
 
+_LOGGER = logging.getLogger(__name__)
+
 
 class Connection:
     """Create connection"""
@@ -100,10 +101,13 @@ class Connection:
     ) -> dict[str, Any]:
         """Run command. Use the existing connection token, otherwise create new one"""
 
+        _LOGGER.debug(f"Started command {command}")
+
         if self._drop:
             return {}
 
         if self._token is None and not retry:
+            _LOGGER.debug(f"No token - connecting and repeating")
             await self.async_connect()
             return await self.async_run_command(command, endpoint, retry=True)
         else:
@@ -111,7 +115,15 @@ class Connection:
                 try:
                     result = await self.async_request(command, endpoint, self._headers)
                     return result
-                except (AsusRouter404, AsusRouterServerDisconnectedError) as ex:
+                except AsusRouterAuthorizationError as ex:
+                    if not retry:
+                        await self.async_connect()
+                        return await self.async_run_command(
+                            command, endpoint, retry=True
+                        )
+                    else:
+                        raise ex
+                except AsusRouterError as ex:
                     raise ex
                 except Exception as ex:
                     if not retry:
@@ -198,13 +210,12 @@ class Connection:
                     error_code = int(json_body["error_status"])
                     # Not authorised
                     if error_code == AR_ERROR["authorisation"]:
-                        raise AsusRouterResponseError(MSG_ERROR["authorisation"])
+                        raise AsusRouterAuthorizationError(MSG_ERROR["authorisation"])
                     # Wrong crerdentials
                     elif error_code == AR_ERROR["credentials"]:
                         raise AsusRouterLoginError(MSG_ERROR["credentials"])
                     # Too many attempts
                     elif error_code == AR_ERROR["try_again"]:
-                        _LOGGER.debug(json_body)
                         raise AsusRouterLoginBlockError(
                             MSG_ERROR["try_again"],
                             timeout=converters.int_from_str(
@@ -213,7 +224,6 @@ class Connection:
                         )
                     # Loged out
                     elif error_code == AR_ERROR["logout"]:
-                        _LOGGER.info(MSG_SUCCESS["logout"])
                         return {"success": True}
                     # Unknown error code
                     else:
@@ -236,10 +246,8 @@ class Connection:
         # Handle non-JSON replies
         except json.JSONDecodeError:
             if ".xml" in endpoint:
-                _LOGGER.debug(MSG_INFO["xml"])
                 json_body = parsers.xml(text=string_body, page=endpoint)
             else:
-                _LOGGER.debug(MSG_INFO["json_fix"])
                 json_body = parsers.pseudo_json(text=string_body, page=endpoint)
             return json_body
 
@@ -258,15 +266,15 @@ class Connection:
             if retry:
                 raise AsusRouterServerDisconnectedError(ex) from ex
             else:
-                _LOGGER.warning(MSG_WARNING["disconnected"].format(endpoint, payload))
+                _LOGGER.debug(MSG_WARNING["disconnected"].format(endpoint, payload))
 
-        # Connection refused -> will repeat
-        except aiohttp.ClientConnectorError as ex:
+        # Connection refused or reset by peer -> will repeat
+        except (aiohttp.ClientOSError) as ex:
             if endpoint == AR_PATH["login"] and not self._mute_flag:
                 raise AsusRouterConnectionError(str(ex)) from ex
             # Mute warning for retries and while connecting
             if not retry and not self._connecting:
-                _LOGGER.info(
+                _LOGGER.debug(
                     "{}. Endpoint: {}. Payload: {}.\nException summary:{}".format(
                         MSG_WARNING["refused"], endpoint, payload, str(ex)
                     )
@@ -277,7 +285,7 @@ class Connection:
 
         if not retry:
             self._error = True
-            _LOGGER.info(MSG_INFO["reconnect"])
+            _LOGGER.debug(MSG_INFO["reconnect"])
             await self.async_cleanup()
             await self.async_connect(retry=True)
         return await self.async_request(
@@ -307,7 +315,7 @@ class Connection:
                 "user-agent": AR_USER_AGENT,
                 "cookie": "asus_token={}".format(self._token),
             }
-            _LOGGER.info(
+            _LOGGER.debug(
                 "{} on port {}: {}".format(
                     MSG_SUCCESS["login"], self._port, self._device
                 )
