@@ -238,9 +238,18 @@ AR_MAP_SYSINFO: dict[str, list[Key]] = {
     ],
 }
 AR_MAP_TEMPERATURE: dict[str, list[str]] = {
-    "2ghz": ['curr_coreTmp_2_raw="([0-9.]+)&deg;C', 'curr_coreTmp_0_raw="([0-9.]+)&deg;C'],
-    "5ghz": ['curr_coreTmp_5_raw="([0-9.]+)&deg;C', 'curr_coreTmp_1_raw="([0-9.]+)&deg;C'],
-    "5ghz2": ['curr_coreTmp_52_raw="([0-9.]+)&deg;C', 'curr_coreTmp_2_raw="([0-9.]+)&deg;C'],
+    "2ghz": [
+        'curr_coreTmp_2_raw="([0-9.]+)&deg;C',
+        'curr_coreTmp_0_raw="([0-9.]+)&deg;C',
+    ],
+    "5ghz": [
+        'curr_coreTmp_5_raw="([0-9.]+)&deg;C',
+        'curr_coreTmp_1_raw="([0-9.]+)&deg;C',
+    ],
+    "5ghz2": [
+        'curr_coreTmp_52_raw="([0-9.]+)&deg;C',
+        'curr_coreTmp_2_raw="([0-9.]+)&deg;C',
+    ],
     "6ghz": ['curr_coreTmp_3_raw="([0-9.]+)&deg;C'],
     "cpu": ['curr_cpuTemp="([0-9.]+)"', 'curr_coreTmp_cpu="([0-9.]+)"'],
 }
@@ -250,7 +259,9 @@ AR_MAP_PARENTAL_CONTROL: dict[str, str] = {
     "MULTIFILTER_MAC": "mac",
     "MULTIFILTER_MACFILTER_DAYTIME_V2": "time",
 }
-AR_KEY_PARENTAL_CONTROL = Key("MULTIFILTER_ALL", "parental_control", method=bool_from_any)
+AR_KEY_PARENTAL_CONTROL = Key(
+    "MULTIFILTER_ALL", "parental_control", method=bool_from_any
+)
 
 AR_PATH = {
     "command": "applyapp.cgi",
